"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const process = require("process");
const error_reporter_1 = require("./shared/error-reporter");
// ts-node required for runtime typescript compilation of sandboxes.ts
require('ts-node/register');
// Legacy import
const asyncMap = require('async/map');
let browser;
let currentScenario = '';
let reporter;
let hostUrl = '';
// Ensure Chromium instances are destroyed on err
process.on('unhandledRejection', () => {
    if (browser)
        browser.close();
});
function verifySandboxes(configuration, sandboxesPath, port) {
    return __awaiter(this, void 0, void 0, function* () {
        hostUrl = `http://localhost:${port}`;
        yield main(configuration, sandboxesPath, port);
    });
}
exports.verifySandboxes = verifySandboxes;
/////////////////////////////////
function main(configuration, sandboxesPath, port) {
    return __awaiter(this, void 0, void 0, function* () {
        const timeoutAttempts = configuration.flags.timeout.value;
        const puppeteer = yield Promise.resolve().then(function () { return require('puppeteer'); });
        browser = yield puppeteer.launch({
            headless: true,
            handleSIGINT: false,
            args: configuration.chromeArguments
        });
        const scenarios = getSandboxMetadata(hostUrl, configuration.flags.randomScenario.value, sandboxesPath);
        reporter = new error_reporter_1.ErrorReporter(scenarios, configuration.flags.reportPath.value, configuration.flags.reportType.value);
        console.log(`Retrieved ${scenarios.length} scenarios.\n`);
        for (let i = 0; i < scenarios.length; i++) {
            console.log(`Checking: ${scenarios[i].name}: ${scenarios[i].description}`);
            yield openScenarioInNewPage(scenarios[i], timeoutAttempts);
        }
        browser.close();
        if (reporter.errors.length > 0) {
            reporter.compileReport();
            process.exit(1);
        }
        else {
            process.exit(0);
        }
    });
}
/**
 * Creates a Chromium page and navigates to a scenario (URL).
 * If Chromium is not able to connect to the provided page, it will issue a series
 * of retries before it finally fails.
 * @param scenario - Scenario to visit
 */
function openScenarioInNewPage(scenario, timeoutAttempts) {
    return __awaiter(this, void 0, void 0, function* () {
        if (timeoutAttempts === 0) {
            yield browser.close();
            throw new Error('Unable to connect to Playground.');
        }
        const page = yield browser.newPage();
        page.on('console', (msg) => onConsoleErr(msg));
        currentScenario = scenario.name;
        try {
            yield page.goto(scenario.url);
        }
        catch (e) {
            yield page.close();
            yield delay(1000);
            yield openScenarioInNewPage(scenario, timeoutAttempts - 1);
        }
    });
}
/**
 * Retrieves Sandbox scenario URLs, descriptions, and names
 * @param baseUrl - Base URL of scenario path e.g. http://localhost:4201
 * @param selectRandomScenario - Whether or not to select one random scenario of all availalble scenarios for a component
 * @param path - Path to sandboxes.ts
 */
function getSandboxMetadata(baseUrl, selectRandomScenario, path) {
    const scenarios = [];
    loadSandboxMenuItems(path).forEach((scenario) => {
        if (selectRandomScenario) {
            const randomItemKey = getRandomKey(scenario.scenarioMenuItems.length);
            scenario.scenarioMenuItems
                .forEach((item) => {
                if (item.key === randomItemKey) {
                    const url = `${baseUrl}?scenario=${encodeURIComponent(scenario.key)}/${encodeURIComponent(item.description)}`;
                    scenarios.push({ url, name: scenario.key, description: item.description });
                }
            });
        }
        else {
            // Grab all scenarios
            scenario.scenarioMenuItems
                .forEach((item) => {
                const url = `${baseUrl}?scenario=${encodeURIComponent(scenario.key)}/${encodeURIComponent(item.description)}`;
                scenarios.push({ url, name: scenario.key, description: item.description });
            });
        }
    });
    return scenarios;
}
/**
 * Attempt to load sandboxes.ts and provide menu items
 * @param path - Path to sandboxes.ts
 */
function loadSandboxMenuItems(path) {
    try {
        return require(path).getSandboxMenuItems();
    }
    catch (err) {
        console.error('Failed to load sandboxes.ts file.');
        console.error(err);
        console.log('Terminating process.');
        process.exit(1);
    }
}
/**
 * Callback when Chromium page encounters a console error
 * @param msg - Error message
 */
function onConsoleErr(msg) {
    return __awaiter(this, void 0, void 0, function* () {
        if (msg.type === 'error') {
            console.error(`${reporter.redWrap('ERROR:')} in ${currentScenario}`);
            const descriptions = msg.args
                .map(a => a._remoteObject)
                .filter(o => o.type === 'object')
                .map(o => o.description);
            descriptions.map(d => console.error(d));
            reporter.addError(descriptions, currentScenario);
        }
    });
}
/**
 * Returns a random value between 1 and the provided length.
 * Note: indexing of keys starts at 1, not 0
 * @param menuItemsLength - Maximum number of items
 */
function getRandomKey(menuItemsLength) {
    return Math.floor(Math.random() * (menuItemsLength - 1) + 1);
}
function delay(ms) {
    return new Promise(resolve => {
        setTimeout(() => {
            resolve();
        }, ms);
    });
}
